/****************************************************************************
** Background timer routines                                               **
**  by Steven H Don                                                        **
*****************************************************************************
** This shows you how to speed up the system timer to specific frequencies **
** and use it do do some useful stuff.                                     **
** NOTE: Windows doesn't like timers at speeds of more than 1000Hz         **
** NOTE: This example program will crash if compiled as is. There is no    **
**       bug in the code, however. If you use the routines in another      **
**       program, they will work correctly. The Pascal version doesn't     **
**       experience this (Turbo C compiler bug?).                          **
**                                                                         **
** For questions, feel free to e-mail me.                                  **
**                                                                         **
**    shd@earthling.net                                                    **
**    http://shd.cjb.net                                                   **
**                                                                         **
****************************************************************************/
#include "dos.h"
#include "conio.h"
#include "stdio.h"

  //This stores a pointer to the original timer handler
  #ifdef __cplusplus
    void interrupt (*BIOSTimerHandler) (...);
  #else
    void interrupt (*BIOSTimerHandler) ();
  #endif
  //This is used for counting the clock ticks
  long ClockTicks, TickCounter;

//This procedure changes the speed of the timer
void ChangeSpeed (long Divisor)
{
  //Set the appropriate values for the timer interrupt
  TickCounter = 0x1234DD / Divisor;
  outportb (0x43, 0x34);
  outportb (0x40, (TickCounter & 0xFF));
  outportb (0x40, (TickCounter >> 8));
}

//This is the new timer interrupt handler
#ifdef __cplusplus
  void interrupt TimerHandler (...)
#else
  void interrupt TimerHandler ()
#endif
{
  //Do whatever you want here, I'm just displaying a character
  printf ("!");
  //Check whether we need to call the original timer handler
  ClockTicks += TickCounter;
  //Do so if required
  if (ClockTicks > 65535) {
    ClockTicks -= 65536;
    asm pushf;
    BIOSTimerHandler ();
  } else
    outportb (0x20, 0x20);
}

//Installs the timer handler
void InstallTimer ()
{
  TickCounter = 65536;
  //Install new timer handler
  setvect (8, TimerHandler);
}

//Restores the BIOS timer handler
void RestoreTimer ()
{
  //Return to 18.2 times a second
  outportb (0x43, 0x34);
  outportb (0x40, 0);
  outportb (0x40, 0);
  //Install old timer handler
  setvect (8, BIOSTimerHandler);
}

void main ()
{
  //Save old timer handler
  BIOSTimerHandler = getvect (8);
  //Do the demo
  clrscr ();
  printf ("Press any key to install new timer handler.\n");
  getch ();
  InstallTimer ();
  printf ("Timer installed.\n");
  getch ();
  printf ("\nIncreasing speed to 100Hz\n");
  ChangeSpeed (100);
  getch ();
  RestoreTimer ();
  printf ("\nTimer restored to BIOS routine.\n");
  getch ();
}